@@ -15,15 +15,17 @@
 import logging
 import os
 
-from paste import deploy
+from oslo import i18n
 
-from keystone.openstack.common import gettextutils
-# NOTE(dstanek): gettextutils.enable_lazy() must be called before
-# gettextutils._() is called to ensure it has the desired lazy lookup
+
+# NOTE(dstanek): i18n.enable_lazy() must be called before
+# keystone.i18n._() is called to ensure it has the desired lazy lookup
 # behavior. This includes cases, like keystone.exceptions, where
-# gettextutils._() is called at import time.
-gettextutils.enable_lazy()
+# keystone.i18n._() is called at import time.
+i18n.enable_lazy()
 
+
+from keystone import backends
 from keystone.common import dependency
 from keystone.common import environment
 from keystone.common import sql
@@ -48,12 +50,11 @@ if CONF.debug:
     CONF.log_opt_values(log.getLogger(CONF.prog), logging.DEBUG)
 
 
-drivers = service.load_backends()
+drivers = backends.load_backends()
 
 # NOTE(ldbragst): 'application' is required in this context by WSGI spec.
 # The following is a reference to Python Paste Deploy documentation
 # http://pythonpaste.org/deploy/
-application = deploy.loadapp('config:%s' % config.find_paste_config(),
-                             name=name)
+application = service.loadapp('config:%s' % config.find_paste_config(), name)
 
 dependency.resolve_future_dependencies()
