@@ -12,6 +12,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from oslo.utils import timeutils
+
 from keystone import auth
 from keystone.common import controller
 from keystone.common import dependency
@@ -20,7 +22,6 @@ from keystone.contrib.oauth1 import validator
 from keystone import exception
 from keystone.i18n import _
 from keystone.openstack.common import log
-from keystone.openstack.common import timeutils
 
 
 LOG = log.getLogger(__name__)
