@@ -20,7 +20,7 @@ from dogpile.cache import util
 
 from keystone import config
 from keystone import exception
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
 from keystone.openstack.common import importutils
 from keystone.openstack.common import log
 
@@ -40,6 +40,11 @@ dogpile.cache.register_backend(
     'keystone.common.cache.backends.mongo',
     'MongoCacheBackend')
 
+dogpile.cache.register_backend(
+    'keystone.cache.memcache_pool',
+    'keystone.common.cache.backends.memcache_pool',
+    'PooledMemcachedBackend')
+
 
 class DebugProxy(proxy.ProxyBackend):
     """Extra Logging ProxyBackend."""
@@ -51,31 +56,31 @@ class DebugProxy(proxy.ProxyBackend):
 
     def get(self, key):
         value = self.proxied.get(key)
-        LOG.debug(_('CACHE_GET: Key: "%(key)r" Value: "%(value)r"'),
+        LOG.debug('CACHE_GET: Key: "%(key)r" Value: "%(value)r"',
                   {'key': key, 'value': value})
         return value
 
     def get_multi(self, keys):
         values = self.proxied.get_multi(keys)
-        LOG.debug(_('CACHE_GET_MULTI: "%(keys)r" Values: "%(values)r"'),
+        LOG.debug('CACHE_GET_MULTI: "%(keys)r" Values: "%(values)r"',
                   {'keys': keys, 'values': values})
         return values
 
     def set(self, key, value):
-        LOG.debug(_('CACHE_SET: Key: "%(key)r" Value: "%(value)r"'),
+        LOG.debug('CACHE_SET: Key: "%(key)r" Value: "%(value)r"',
                   {'key': key, 'value': value})
         return self.proxied.set(key, value)
 
     def set_multi(self, keys):
-        LOG.debug(_('CACHE_SET_MULTI: "%r"'), keys)
+        LOG.debug('CACHE_SET_MULTI: "%r"', keys)
         self.proxied.set_multi(keys)
 
     def delete(self, key):
         self.proxied.delete(key)
-        LOG.debug(_('CACHE_DELETE: "%r"'), key)
+        LOG.debug('CACHE_DELETE: "%r"', key)
 
     def delete_multi(self, keys):
-        LOG.debug(_('CACHE_DELETE_MULTI: "%r"'), keys)
+        LOG.debug('CACHE_DELETE_MULTI: "%r"', keys)
         self.proxied.delete_multi(keys)
 
 
@@ -101,7 +106,16 @@ def build_cache_config():
         arg_key = '.'.join([prefix, 'arguments', argname])
         conf_dict[arg_key] = argvalue
 
-        LOG.debug(_('Keystone Cache Config: %s'), conf_dict)
+        LOG.debug('Keystone Cache Config: %s', conf_dict)
+    # NOTE(yorik-sar): these arguments will be used for memcache-related
+    # backends. Use setdefault for url to support old-style setting through
+    # backend_argument=url:127.0.0.1:11211
+    conf_dict.setdefault('%s.arguments.url' % prefix,
+                         CONF.cache.memcache_servers)
+    for arg in ('dead_retry', 'socket_timeout', 'pool_maxsize',
+                'pool_unused_timeout', 'pool_connection_get_timeout'):
+        value = getattr(CONF.cache, 'memcache_' + arg)
+        conf_dict['%s.arguments.%s' % (prefix, arg)] = value
 
     return conf_dict
 
@@ -118,7 +132,7 @@ def configure_cache_region(region):
         raise exception.ValidationError(
             _('region not type dogpile.cache.CacheRegion'))
 
-    if 'backend' not in region.__dict__:
+    if not region.is_configured:
         # NOTE(morganfainberg): this is how you tell if a region is configured.
         # There is a request logged with dogpile.cache upstream to make this
         # easier / less ugly.
@@ -134,12 +148,9 @@ def configure_cache_region(region):
         # key_mangler, we should respect that key_mangler function.  If a
         # key_mangler is not defined by the backend, use the sha1_mangle_key
         # mangler provided by dogpile.cache. This ensures we always use a fixed
-        # size cache-key.  This is toggle-able for debug purposes; if disabled
-        # this could cause issues with certain backends (such as memcached) and
-        # its limited key-size.
+        # size cache-key.
         if region.key_mangler is None:
-            if CONF.cache.use_key_mangler:
-                region.key_mangler = util.sha1_mangle_key
+            region.key_mangler = util.sha1_mangle_key
 
         for class_path in CONF.cache.proxies:
             # NOTE(morganfainberg): if we have any proxy wrappers, we should
@@ -149,7 +160,7 @@ def configure_cache_region(region):
             # ProxyBackends work, see the dogpile.cache documents on
             # "changing-backend-behavior"
             cls = importutils.import_class(class_path)
-            LOG.debug(_("Adding cache-proxy '%s' to backend."), class_path)
+            LOG.debug("Adding cache-proxy '%s' to backend.", class_path)
             region.wrap(cls)
 
     return region
