@@ -98,7 +98,7 @@ import re
 import six
 
 from keystone.common import base64utils
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
 
 
 PEM_TYPE_TO_HEADER = {
@@ -250,8 +250,6 @@ class PEMParseResult(object):
             self._pem_type = pem_type
             self._pem_header = pem_header
 
-#------------------------------------------------------------------------------
-
 
 def pem_search(text, start=0):
     """Search for a block of PEM formatted data
@@ -321,7 +319,7 @@ def parse_pem(text, pem_type=None, max_items=None):
     """Scan text for PEM data, return list of PEM items
 
     The input text is scanned for PEM blocks, for each one found a
-    PEMParseResult is contructed and added to the return list.
+    PEMParseResult is constructed and added to the return list.
 
     pem_type operates as a filter on the type of PEM desired. If
     pem_type is specified only those PEM blocks which match will be
