@@ -11,49 +11,16 @@
 # under the License.
 """Internal implementation of request body validating middleware."""
 
-import re
-
 import jsonschema
-from oslo_config import cfg
-from oslo_log import log
 
 from keystone import exception
 from keystone.i18n import _
 
-CONF = cfg.CONF
-LOG = log.getLogger(__name__)
 
-
-# TODO(rderose): extend schema validation and add this check there
-def validate_password(password):
-    pattern = CONF.security_compliance.password_regex
-    if pattern:
-        if not isinstance(password, str):
-            detail = _("Password must be a string type")
-            raise exception.PasswordValidationError(detail=detail)
-        try:
-            if not re.match(pattern, password):
-                pattern_desc = (
-                    CONF.security_compliance.password_regex_description
-                )
-                raise exception.PasswordRequirementsValidationError(
-                    detail=pattern_desc
-                )
-        except re.error:
-            msg = (
-                "Unable to validate password due to invalid regular "
-                "expression - password_regex: %s"
-            )
-            LOG.error(msg, pattern)
-            detail = _(
-                "Unable to validate password due to invalid configuration"
-            )
-            raise exception.PasswordValidationError(detail=detail)
-
-
-class SchemaValidator:
+class SchemaValidator(object):
     """Resource reference validator class."""
 
+    validator = None
     validator_org = jsonschema.Draft4Validator
 
     def __init__(self, schema):
@@ -65,9 +32,8 @@ class SchemaValidator:
         # didn't by default. See the Nova V3 validator for details on how this
         # is done.
         validators = {}
-        validator_cls = jsonschema.validators.extend(
-            self.validator_org, validators
-        )
+        validator_cls = jsonschema.validators.extend(self.validator_org,
+                                                     validators)
         fc = jsonschema.FormatChecker()
         self.validator = validator_cls(schema, format_checker=fc)
 
@@ -77,7 +43,7 @@ class SchemaValidator:
         except jsonschema.ValidationError as ex:
             # NOTE: For whole OpenStack message consistency, this error
             # message has been written in a format consistent with WSME.
-            if ex.path:
+            if len(ex.path) > 0:
                 # NOTE(lbragstad): Here we could think about using iter_errors
                 # as a method of providing invalid parameters back to the
                 # user.
@@ -85,10 +51,9 @@ class SchemaValidator:
                 # too long, then we should build the masking in here so that
                 # we don't expose sensitive user information in the event it
                 # fails validation.
-                path = '/'.join(map(str, ex.path))
-                detail = _(
-                    "Invalid input for field '%(path)s': %(message)s"
-                ) % {'path': path, 'message': str(ex)}
+                detail = _("Invalid input for field '%(path)s'. The value is "
+                           "'%(value)s'.") % {'path': ex.path.pop(),
+                                              'value': ex.instance}
             else:
-                detail = str(ex)
+                detail = ex.message
             raise exception.SchemaValidationError(detail=detail)
