@@ -16,10 +16,14 @@ from keystone.auth import controllers as auth_controllers
 from keystone.common import authorization
 from keystone.common import controller
 from keystone.common import dependency
+from keystone.common import validation
 from keystone.common import wsgi
 from keystone import config
+from keystone.contrib.federation import idp as keystone_idp
+from keystone.contrib.federation import schema
 from keystone.contrib.federation import utils
 from keystone import exception
+from keystone.models import token_model
 
 
 CONF = config.CONF
@@ -75,20 +79,6 @@ class IdentityProvider(_ControllerBase):
         ref = cls.filter_params(ref)
         return {cls.member_name: ref}
 
-    #TODO(marek-denis): Implement, when mapping engine is ready
-    def _delete_tokens_issued_by_idp(self, idp_id):
-        """Delete tokens created upon authentication from an IdP
-
-        After the IdP is deregistered, users authenticating via such IdP should
-        no longer be allowed to use federated services. Thus, delete all the
-        tokens issued upon authentication from IdP with idp_id id
-
-        :param idp_id: id of Identity Provider for which related tokens should
-                       be removed.
-
-        """
-        raise exception.NotImplemented()
-
     @controller.protected()
     def create_identity_provider(self, context, idp_id, identity_provider):
         identity_provider = self._normalize_dict(identity_provider)
@@ -254,8 +244,8 @@ class Auth(auth_controllers.Auth):
         """
         auth = {
             'identity': {
-                'methods': ['saml2'],
-                'saml2': {
+                'methods': [protocol],
+                protocol: {
                     'identity_provider': identity_provider,
                     'protocol': protocol
                 }
@@ -264,6 +254,39 @@ class Auth(auth_controllers.Auth):
 
         return self.authenticate_for_token(context, auth=auth)
 
+    @validation.validated(schema.saml_create, 'auth')
+    def create_saml_assertion(self, context, auth):
+        """Exchange a scoped token for a SAML assertion.
+
+        :param auth: Dictionary that contains a token id and region id
+        :returns: SAML Assertion based on properties from the token
+        """
+
+        issuer = CONF.saml.idp_entity_id
+        region_id = auth['scope']['region']['id']
+        region = self.catalog_api.get_region(region_id)
+        recipient = region['url']
+
+        token_id = auth['identity']['token']['id']
+        token_data = self.token_provider_api.validate_token(token_id)
+        token_ref = token_model.KeystoneToken(token_id, token_data)
+        subject = token_ref.user_name
+        roles = token_ref.role_names
+
+        if token_ref.project_scoped:
+            project = token_ref.project_name
+        else:
+            raise ValueError(_('Use a project scoped token when attempting to'
+                               'create a SAML assertion'))
+
+        generator = keystone_idp.SAMLGenerator()
+        response = generator.samlize_token(issuer, recipient, subject, roles,
+                                           project)
+
+        return wsgi.render_response(body=response.to_string(),
+                                    status=('200', 'OK'),
+                                    headers=[('Content-Type', 'text/xml')])
+
 
 @dependency.requires('assignment_api')
 class DomainV3(controller.V3Controller):
@@ -309,3 +332,18 @@ class ProjectV3(controller.V3Controller):
         projects = self.assignment_api.list_projects_for_groups(
             auth_context['group_ids'])
         return ProjectV3.wrap_collection(context, projects)
+
+
+class SAMLMetadataV3(_ControllerBase):
+    member_name = 'metadata'
+
+    def get_metadata(self, context):
+        metadata_path = CONF.saml.idp_metadata_path
+        try:
+            with open(metadata_path, 'r') as metadata_handler:
+                metadata = metadata_handler.read()
+        except IOError as e:
+            # Raise HTTP 500 in case Metadata file cannot be read.
+            raise exception.MetadataFileError(reason=e)
+        return wsgi.render_response(body=metadata, status=('200', 'OK'),
+                                    headers=[('Content-Type', 'text/xml')])
