@@ -12,12 +12,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from oslo.utils import encodeutils
 import six
 
 from keystone.common import config
-from keystone.openstack.common.gettextutils import _ # flake8: noqa
+from keystone.i18n import _
 from keystone.openstack.common import log
-from keystone.openstack.common import strutils
 
 
 CONF = config.CONF
@@ -62,7 +62,7 @@ class Error(Exception):
                 message = self.message_format % kwargs
             except UnicodeDecodeError:
                 try:
-                    kwargs = dict([(k, strutils.safe_decode(v)) for k, v in
+                    kwargs = dict([(k, encodeutils.safe_decode(v)) for k, v in
                                    six.iteritems(kwargs)])
                 except UnicodeDecodeError:
                     # NOTE(jamielennox): This is the complete failure case
@@ -76,14 +76,20 @@ class Error(Exception):
 
 
 class ValidationError(Error):
-    message_format = _("Expecting to find %(attribute)s in %(target)s."
-                       " The server could not comply with the request"
+    message_format = _("Expecting to find %(attribute)s in %(target)s -"
+                       " the server could not comply with the request"
                        " since it is either malformed or otherwise"
                        " incorrect. The client is assumed to be in error.")
     code = 400
     title = 'Bad Request'
 
 
+class SchemaValidationError(ValidationError):
+    # NOTE(lbragstad): For whole OpenStack message consistency, this error
+    # message has been written in a format consistent with WSME.
+    message_format = _("%(detail)s")
+
+
 class ValidationTimeStampError(Error):
     message_format = _("Timestamp not in expected format."
                        " The server could not comply with the request"
@@ -109,6 +115,21 @@ class ValidationSizeError(Error):
     title = 'Bad Request'
 
 
+class PasswordVerificationError(Error):
+    message_format = _("The password length must be less than or equal "
+                       "to %(size)i. The server could not comply with the "
+                       "request because the password is invalid.")
+    code = 403
+    title = 'Forbidden'
+
+
+class RegionDeletionError(Error):
+    message_format = _("Unable to delete region %(region_id)s because it or "
+                       "its child regions have associated endpoints.")
+    code = 403
+    title = 'Forbidden'
+
+
 class PKITokenExpected(Error):
     message_format = _('The certificates you requested are not available. '
                        'It is likely that this server does not use PKI tokens '
@@ -119,11 +140,14 @@ class PKITokenExpected(Error):
 
 class SecurityError(Error):
     """Avoids exposing details of security failures, unless in debug mode."""
+    amendment = _('(Disable debug mode to suppress these details.)')
 
     def _build_message(self, message, **kwargs):
         """Only returns detailed messages in debug mode."""
         if CONF.debug:
-            return message or self.message_format % kwargs
+            return _('%(message)s %(amendment)s') % {
+                'message': message or self.message_format % kwargs,
+                'amendment': self.amendment}
         else:
             return self.message_format % kwargs
 
@@ -172,22 +196,35 @@ class Forbidden(SecurityError):
 
 class ForbiddenAction(Forbidden):
     message_format = _("You are not authorized to perform the"
-                       " requested action, %(action)s.")
+                       " requested action: %(action)s")
 
 
 class ImmutableAttributeError(Forbidden):
-    message_format = _("Could not change immutable attribute %(attribute)s"
-                       " in target %(target)s")
+    message_format = _("Could not change immutable attribute(s) "
+                       "'%(attributes)s' in target %(target)s")
+
+
+class CrossBackendNotAllowed(Forbidden):
+    message_format = _("Group membership across backend boundaries is not "
+                       "allowed, group in question is %(group_id)s, "
+                       "user is %(user_id)s")
+
+
+class InvalidPolicyAssociation(Forbidden):
+    message_format = _("Invalid mix of entities for policy association - "
+                       "only Endpoint, Service or Region+Service allowed. "
+                       "Request was - Endpoint: %(endpoint_id)s, "
+                       "Service: %(service_id)s, Region: %(region_id)s")
 
 
 class NotFound(Error):
-    message_format = _("Could not find, %(target)s.")
+    message_format = _("Could not find: %(target)s")
     code = 404
     title = 'Not Found'
 
 
 class EndpointNotFound(NotFound):
-    message_format = _("Could not find endpoint, %(endpoint_id)s.")
+    message_format = _("Could not find endpoint: %(endpoint_id)s")
 
 
 class MetadataNotFound(NotFound):
@@ -199,72 +236,86 @@ class MetadataNotFound(NotFound):
 
 
 class PolicyNotFound(NotFound):
-    message_format = _("Could not find policy, %(policy_id)s.")
+    message_format = _("Could not find policy: %(policy_id)s")
+
+
+class PolicyAssociationNotFound(NotFound):
+    message_format = _("Could not find policy association")
 
 
 class RoleNotFound(NotFound):
-    message_format = _("Could not find role, %(role_id)s.")
+    message_format = _("Could not find role: %(role_id)s")
 
 
 class RegionNotFound(NotFound):
-    message_format = _("Could not find region, %(region_id)s.")
+    message_format = _("Could not find region: %(region_id)s")
 
 
 class ServiceNotFound(NotFound):
-    message_format = _("Could not find service, %(service_id)s.")
+    message_format = _("Could not find service: %(service_id)s")
 
 
 class DomainNotFound(NotFound):
-    message_format = _("Could not find domain, %(domain_id)s.")
+    message_format = _("Could not find domain: %(domain_id)s")
 
 
 class ProjectNotFound(NotFound):
-    message_format = _("Could not find project, %(project_id)s.")
+    message_format = _("Could not find project: %(project_id)s")
 
 
 class TokenNotFound(NotFound):
-    message_format = _("Could not find token, %(token_id)s.")
+    message_format = _("Could not find token: %(token_id)s")
 
 
 class UserNotFound(NotFound):
-    message_format = _("Could not find user, %(user_id)s.")
+    message_format = _("Could not find user: %(user_id)s")
 
 
 class GroupNotFound(NotFound):
-    message_format = _("Could not find group, %(group_id)s.")
+    message_format = _("Could not find group: %(group_id)s")
 
 
 class MappingNotFound(NotFound):
-    message_format = _("Could not find mapping, %(mapping_id)s.")
+    message_format = _("Could not find mapping: %(mapping_id)s")
 
 
 class TrustNotFound(NotFound):
-    message_format = _("Could not find trust, %(trust_id)s.")
+    message_format = _("Could not find trust: %(trust_id)s")
 
 
 class TrustUseLimitReached(Forbidden):
-    message_format = _("No remaining uses for trust %(trust_id)s.")
+    message_format = _("No remaining uses for trust: %(trust_id)s")
 
 
 class CredentialNotFound(NotFound):
-    message_format = _("Could not find credential, %(credential_id)s.")
+    message_format = _("Could not find credential: %(credential_id)s")
 
 
 class VersionNotFound(NotFound):
-    message_format = _("Could not find version, %(version)s.")
+    message_format = _("Could not find version: %(version)s")
+
+
+class EndpointGroupNotFound(NotFound):
+    message_format = _("Could not find Endpoint Group: %(endpoint_group_id)s")
 
 
 class IdentityProviderNotFound(NotFound):
-    message_format = _("Could not find IdentityProvider, %(idp_id)s.")
+    message_format = _("Could not find Identity Provider: %(idp_id)s")
 
 
 class FederatedProtocolNotFound(NotFound):
     message_format = _("Could not find federated protocol %(protocol_id)s for"
-                       " IdentityProvider, %(idp_id)s")
+                       " Identity Provider: %(idp_id)s")
+
+
+class PublicIDNotFound(NotFound):
+    # This is used internally and mapped to either User/GroupNotFound or,
+    # Assertion before the exception leaves Keystone.
+    message_format = "%(id)s"
 
 
 class Conflict(Error):
-    message_format = _("Conflict occurred attempting to store %(type)s."
+    message_format = _("Conflict occurred attempting to store %(type)s -"
                        " %(details)s")
     code = 409
     title = 'Conflict'
@@ -282,7 +333,7 @@ class UnexpectedError(SecurityError):
                         "from fulfilling your request.")
 
     debug_message_format = _("An unexpected error prevented the server "
-                             "from fulfilling your request. %(exception)s")
+                             "from fulfilling your request: %(exception)s")
 
     @property
     def message_format(self):
@@ -324,6 +375,10 @@ class MappedGroupNotFound(UnexpectedError):
                              "%(mapping_id)s was not found in the backend.")
 
 
+class MetadataFileError(UnexpectedError):
+    message_format = _("Error while reading metadata file, %(reason)s")
+
+
 class NotImplemented(Error):
     message_format = _("The action you have requested has not"
                        " been implemented.")
@@ -343,9 +398,28 @@ class ConfigFileNotFound(UnexpectedError):
                              "could not be found.")
 
 
+class MultipleSQLDriversInConfig(UnexpectedError):
+    message_format = _('The Keystone domain configuration file '
+                       '%(config_file)s defines an additional SQL driver - '
+                       'only one is permitted.')
+
+
 class MigrationNotProvided(Exception):
     def __init__(self, mod_name, path):
         super(MigrationNotProvided, self).__init__(_(
             "%(mod_name)s doesn't provide database migrations. The migration"
             " repository path at %(path)s doesn't exist or isn't a directory."
         ) % {'mod_name': mod_name, 'path': path})
+
+
+class UnsupportedTokenVersionException(Exception):
+    """Token version is unrecognizable or unsupported."""
+    pass
+
+
+class SAMLSigningError(UnexpectedError):
+    debug_message_format = _('Unable to sign SAML assertion. It is likely '
+                             'that this server does not have xmlsec1 '
+                             'installed, or this is the result of '
+                             'misconfiguration. Reason %(reason)s')
+    title = 'Error signing SAML assertion'
