@@ -21,7 +21,8 @@ from keystone.common import serializer
 from keystone.common import utils
 from keystone.common import wsgi
 from keystone import exception
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
+from keystone.models import token_model
 from keystone.openstack.common import jsonutils
 from keystone.openstack.common import log
 from keystone.openstack.common import versionutils
@@ -131,6 +132,11 @@ class JsonBodyMiddleware(wsgi.Middleware):
             if not params_parsed:
                 params_parsed = {}
 
+        if not isinstance(params_parsed, dict):
+            e = exception.ValidationError(attribute='valid JSON object',
+                                          target='request body')
+            return wsgi.render_exception(e, request=request)
+
         params = {}
         for k, v in six.iteritems(params_parsed):
             if k in ('self', 'context'):
@@ -220,13 +226,13 @@ class RequestBodySizeLimiter(wsgi.Middleware):
 
     @webob.dec.wsgify()
     def __call__(self, req):
-
-        if req.content_length > CONF.max_request_body_size:
+        if req.content_length is None:
+            if req.is_body_readable:
+                limiter = utils.LimitingReader(req.body_file,
+                                               CONF.max_request_body_size)
+                req.body_file = limiter
+        elif req.content_length > CONF.max_request_body_size:
             raise exception.RequestTooLarge()
-        if req.content_length is None and req.is_body_readable:
-            limiter = utils.LimitingReader(req.body_file,
-                                           CONF.max_request_body_size)
-            req.body_file = limiter
         return self.application
 
 
@@ -234,7 +240,7 @@ class AuthContextMiddleware(wsgi.Middleware):
     """Build the authentication context from the request auth token."""
 
     def _build_auth_context(self, request):
-        token_id = request.headers.get(AUTH_TOKEN_HEADER)
+        token_id = request.headers.get(AUTH_TOKEN_HEADER).strip()
 
         if token_id == CONF.admin_token:
             # NOTE(gyee): no need to proceed any further as the special admin
@@ -248,28 +254,21 @@ class AuthContextMiddleware(wsgi.Middleware):
         context['environment'] = request.environ
 
         try:
-            token_ref = self.token_api.get_token(token_id)
-            # TODO(ayoung): These two functions return the token in different
-            # formats instead of two calls, only make one.  However, the call
-            # to get_token hits the caching layer, and does not validate the
-            # token.  In the future, this should be reduced to one call.
-            if not CONF.token.revoke_by_id:
-                self.token_api.token_provider_api.validate_token(
-                    context['token_id'])
-
+            token_ref = token_model.KeystoneToken(
+                token_id=token_id,
+                token_data=self.token_provider_api.validate_token(token_id))
             # TODO(gyee): validate_token_bind should really be its own
             # middleware
             wsgi.validate_token_bind(context, token_ref)
-            return authorization.token_to_auth_context(
-                token_ref['token_data'])
+            return authorization.token_to_auth_context(token_ref)
         except exception.TokenNotFound:
             LOG.warning(_('RBAC: Invalid token'))
             raise exception.Unauthorized()
 
     def process_request(self, request):
         if AUTH_TOKEN_HEADER not in request.headers:
-            LOG.debug(_('Auth token not in the request header. '
-                        'Will not build auth context.'))
+            LOG.debug(('Auth token not in the request header. '
+                       'Will not build auth context.'))
             return
 
         if authorization.AUTH_CONTEXT_ENV in request.environ:
@@ -278,5 +277,5 @@ class AuthContextMiddleware(wsgi.Middleware):
             return
 
         auth_context = self._build_auth_context(request)
-        LOG.debug(_('RBAC: auth_context: %s'), auth_context)
+        LOG.debug('RBAC: auth_context: %s', auth_context)
         request.environ[authorization.AUTH_CONTEXT_ENV] = auth_context
