@@ -23,6 +23,7 @@ from keystone.common import serializer
 from keystone.openstack.common import jsonutils
 from keystone import tests
 from keystone.tests import default_fixtures
+from keystone.tests.ksfixtures import database
 
 
 class RestfulTestCase(tests.TestCase):
@@ -61,6 +62,7 @@ class RestfulTestCase(tests.TestCase):
         # Will need to reset the plug-ins
         self.addCleanup(setattr, auth_controllers, 'AUTH_METHODS', {})
 
+        self.useFixture(database.Database())
         self.load_backends()
         self.load_fixtures(default_fixtures)
 
@@ -103,7 +105,7 @@ class RestfulTestCase(tests.TestCase):
 
         example::
 
-             self.assertResponseSuccessful(response, 203)
+             self.assertResponseSuccessful(response)
         """
         self.assertTrue(
             response.status_code >= 200 and response.status_code <= 299,
@@ -118,7 +120,7 @@ class RestfulTestCase(tests.TestCase):
 
         example::
 
-            self.assertResponseStatus(response, 203)
+            self.assertResponseStatus(response, 204)
         """
         self.assertEqual(
             response.status_code,
@@ -169,9 +171,12 @@ class RestfulTestCase(tests.TestCase):
                 response.result = jsonutils.loads(response.body)
             elif content_type == 'xml':
                 response.result = etree.fromstring(response.body)
+            else:
+                response.result = response.body
 
     def restful_request(self, method='GET', headers=None, body=None,
-                        content_type=None, **kwargs):
+                        content_type=None, response_content_type=None,
+                        **kwargs):
         """Serializes/deserializes json/xml as request/response body.
 
         .. WARNING::
@@ -189,7 +194,8 @@ class RestfulTestCase(tests.TestCase):
         response = self.request(method=method, headers=headers, body=body,
                                 **kwargs)
 
-        self._from_content_type(response, content_type)
+        response_content_type = response_content_type or content_type
+        self._from_content_type(response, content_type=response_content_type)
 
         # we can save some code & improve coverage by always doing this
         if method != 'HEAD' and response.status_code >= 400:
