@@ -17,7 +17,7 @@ import uuid
 
 from keystone import catalog
 from keystone import tests
-from keystone.tests import default_fixtures
+from keystone.tests.ksfixtures import database
 from keystone.tests import test_v3
 
 
@@ -78,6 +78,17 @@ class CatalogTestCase(test_v3.RestfulTestCase):
                 'region_id': ref['id']})
         self.assertValidRegionResponse(r, ref)
 
+    def test_create_region_with_empty_id(self):
+        """Call ``POST /regions`` with an empty ID in the request body."""
+        ref = self.new_region_ref()
+        ref['id'] = ''
+
+        r = self.post(
+            '/regions',
+            body={'region': ref}, expected_status=201)
+        self.assertValidRegionResponse(r, ref)
+        self.assertNotEmpty(r.result['region'].get('id'))
+
     def test_create_region_without_id(self):
         """Call ``POST /regions`` without an ID in the request body."""
         ref = self.new_region_ref()
@@ -92,6 +103,76 @@ class CatalogTestCase(test_v3.RestfulTestCase):
             expected_status=201)
         self.assertValidRegionResponse(r, ref)
 
+    def test_create_region_without_description(self):
+        """Call ``POST /regions`` without description in the request body."""
+        ref = self.new_region_ref()
+
+        del ref['description']
+
+        r = self.post(
+            '/regions',
+            body={'region': ref},
+            expected_status=201)
+        # Create the description in the reference to compare to since the
+        # response should now have a description, even though we didn't send
+        # it with the original reference.
+        ref['description'] = ''
+        self.assertValidRegionResponse(r, ref)
+
+    def test_create_regions_with_same_description_string(self):
+        """Call ``POST /regions`` with same description in the request bodies.
+        """
+        # NOTE(lbragstad): Make sure we can create two regions that have the
+        # same description.
+        ref1 = self.new_region_ref()
+        ref2 = self.new_region_ref()
+
+        region_desc = 'Some Region Description'
+
+        ref1['description'] = region_desc
+        ref2['description'] = region_desc
+
+        resp1 = self.post(
+            '/regions',
+            body={'region': ref1},
+            expected_status=201)
+        self.assertValidRegionResponse(resp1, ref1)
+
+        resp2 = self.post(
+            '/regions',
+            body={'region': ref2},
+            expected_status=201)
+        self.assertValidRegionResponse(resp2, ref2)
+
+    def test_create_regions_without_descriptions(self):
+        """Call ``POST /regions`` with no description in the request bodies.
+        """
+        # NOTE(lbragstad): Make sure we can create two regions that have
+        # no description in the request body. The description should be
+        # populated by Catalog Manager.
+        ref1 = self.new_region_ref()
+        ref2 = self.new_region_ref()
+
+        del ref1['description']
+        del ref2['description']
+
+        resp1 = self.post(
+            '/regions',
+            body={'region': ref1},
+            expected_status=201)
+
+        resp2 = self.post(
+            '/regions',
+            body={'region': ref2},
+            expected_status=201)
+        # Create the descriptions in the references to compare to since the
+        # responses should now have descriptions, even though we didn't send
+        # a description with the original references.
+        ref1['description'] = ''
+        ref2['description'] = ''
+        self.assertValidRegionResponse(resp1, ref1)
+        self.assertValidRegionResponse(resp2, ref2)
+
     def test_create_region_with_conflicting_ids(self):
         """Call ``PUT /regions/{region_id}`` with conflicting region IDs."""
         # the region ref is created with an ID
@@ -108,6 +189,26 @@ class CatalogTestCase(test_v3.RestfulTestCase):
         r = self.get('/regions')
         self.assertValidRegionListResponse(r, ref=self.region)
 
+    def _create_region_with_parent_id(self, parent_id=None):
+        ref = self.new_region_ref()
+        ref['parent_region_id'] = parent_id
+        return self.post(
+            '/regions',
+            body={'region': ref})
+
+    def test_list_regions_filtered_by_parent_region_id(self):
+        """Call ``GET /regions?parent_region_id={parent_region_id}``."""
+        new_region = self._create_region_with_parent_id()
+        parent_id = new_region.result['region']['id']
+
+        new_region = self._create_region_with_parent_id(parent_id)
+        new_region = self._create_region_with_parent_id(parent_id)
+
+        r = self.get('/regions?parent_region_id=%s' % parent_id)
+
+        for region in r.result['regions']:
+            self.assertEqual(parent_id, region['parent_region_id'])
+
     def test_list_regions_xml(self):
         """Call ``GET /regions (xml data)``."""
         r = self.get('/regions', content_type='xml')
@@ -130,8 +231,57 @@ class CatalogTestCase(test_v3.RestfulTestCase):
 
     def test_delete_region(self):
         """Call ``DELETE /regions/{region_id}``."""
+
+        ref = self.new_region_ref()
+        r = self.post(
+            '/regions',
+            body={'region': ref})
+        self.assertValidRegionResponse(r, ref)
+
         self.delete('/regions/%(region_id)s' % {
-            'region_id': self.region_id})
+            'region_id': ref['id']})
+
+    def test_create_region_with_url(self):
+        """Call ``POST /regions`` with a custom url field."""
+        ref = self.new_region_ref()
+        ref['url'] = 'http://beta.com:5000/v3'
+
+        r = self.post(
+            '/regions',
+            body={'region': ref},
+            expected_status=201)
+
+        self.assertEqual(ref['url'], r.json['region']['url'])
+        self.assertValidRegionResponse(r, ref)
+
+        r = self.get(
+            '/regions/%(region_id)s' % {
+                'region_id': ref['id']})
+        self.assertEqual(ref['url'], r.json['region']['url'])
+        self.assertValidRegionResponse(r, ref)
+
+    def test_update_region_with_url(self):
+        """Call ``PUT /regions`` with a custom url field."""
+        ref = self.new_region_ref()
+
+        r = self.post(
+            '/regions',
+            body={'region': ref},
+            expected_status=201)
+
+        self.assertIsNone(r.json['region']['url'])
+        self.assertValidRegionResponse(r, ref)
+
+        region_id = r.json['region']['id']
+        ref = self.new_region_ref()
+        del ref['id']
+        ref['url'] = 'http://beta.com:5000/v3'
+
+        r = self.patch('/regions/%(region_id)s' % {
+            'region_id': region_id},
+            body={'region': ref})
+        self.assertEqual(ref['url'], r.json['region']['url'])
+        self.assertValidRegionResponse(r, ref)
 
     # service crud tests
 
@@ -197,6 +347,48 @@ class CatalogTestCase(test_v3.RestfulTestCase):
         r = self.get('/services')
         self.assertValidServiceListResponse(r, ref=self.service)
 
+    def _create_random_service(self):
+        ref = self.new_service_ref()
+        ref['enabled'] = True
+        response = self.post(
+            '/services',
+            body={'service': ref})
+        return response.json['service']
+
+    def test_filter_list_services_by_type(self):
+        """Call ``GET /services?type=<some type>``."""
+        target_ref = self._create_random_service()
+
+        # create unrelated services
+        self._create_random_service()
+        self._create_random_service()
+
+        response = self.get('/services?type=' + target_ref['type'])
+        self.assertValidServiceListResponse(response, ref=target_ref)
+
+        filtered_service_list = response.json['services']
+        self.assertEqual(1, len(filtered_service_list))
+
+        filtered_service = filtered_service_list[0]
+        self.assertEqual(target_ref['type'], filtered_service['type'])
+
+    def test_filter_list_services_by_name(self):
+        """Call ``GET /services?name=<some name>``."""
+        target_ref = self._create_random_service()
+
+        # create unrelated services
+        self._create_random_service()
+        self._create_random_service()
+
+        response = self.get('/services?name=' + target_ref['name'])
+        self.assertValidServiceListResponse(response, ref=target_ref)
+
+        filtered_service_list = response.json['services']
+        self.assertEqual(1, len(filtered_service_list))
+
+        filtered_service = filtered_service_list[0]
+        self.assertEqual(target_ref['name'], filtered_service['name'])
+
     def test_list_services_xml(self):
         """Call ``GET /services (xml data)``."""
         r = self.get('/services', content_type='xml')
@@ -288,19 +480,34 @@ class CatalogTestCase(test_v3.RestfulTestCase):
             body={'endpoint': ref},
             expected_status=400)
 
-    def assertValidErrorResponse(self, response):
-        self.assertTrue(response.status_code in [400, 409])
-
-    def test_create_endpoint_400(self):
+    def test_create_endpoint_with_invalid_region_id(self):
         """Call ``POST /endpoints``."""
         ref = self.new_endpoint_ref(service_id=self.service_id)
-        ref["region"] = "0" * 256
+        ref["region_id"] = uuid.uuid4().hex
         self.post('/endpoints', body={'endpoint': ref}, expected_status=400)
 
+    def test_create_endpoint_with_region(self):
+        """EndpointV3 creates the region before creating the endpoint, if
+        endpoint is provided with 'region' and no 'region_id'
+        """
+        ref = self.new_endpoint_ref(service_id=self.service_id)
+        ref["region"] = uuid.uuid4().hex
+        ref.pop('region_id')
+        self.post('/endpoints', body={'endpoint': ref}, expected_status=201)
+        # Make sure the region is created
+        self.get('/regions/%(region_id)s' % {
+            'region_id': ref["region"]})
+
+    def test_create_endpoint_with_no_region(self):
+        """EndpointV3 allows to creates the endpoint without region."""
+        ref = self.new_endpoint_ref(service_id=self.service_id)
+        ref.pop('region_id')
+        self.post('/endpoints', body={'endpoint': ref}, expected_status=201)
+
     def test_create_endpoint_with_empty_url(self):
         """Call ``POST /endpoints``."""
         ref = self.new_endpoint_ref(service_id=self.service_id)
-        del ref["url"]
+        ref["url"] = ''
         self.post('/endpoints', body={'endpoint': ref}, expected_status=400)
 
     def test_get_endpoint(self):
@@ -381,6 +588,8 @@ class CatalogTestCase(test_v3.RestfulTestCase):
         del ref['interface']
         ref['publicurl'] = ref.pop('url')
         ref['internalurl'] = None
+        ref['region'] = ref['region_id']
+        del ref['region_id']
         # don't set adminurl to ensure it's absence is handled like internalurl
 
         # create the endpoint on v2 (using a v3 token)
@@ -394,21 +603,21 @@ class CatalogTestCase(test_v3.RestfulTestCase):
         # test the endpoint on v3
         r = self.get('/endpoints')
         endpoints = self.assertValidEndpointListResponse(r)
-        self.assertEqual(len(endpoints), 1)
+        self.assertEqual(1, len(endpoints))
         endpoint_v3 = endpoints.pop()
 
-        # these attributes are identical between both API's
-        self.assertEqual(endpoint_v3['region'], ref['region'])
-        self.assertEqual(endpoint_v3['service_id'], ref['service_id'])
-        self.assertEqual(endpoint_v3['description'], ref['description'])
+        # these attributes are identical between both APIs
+        self.assertEqual(ref['region'], endpoint_v3['region_id'])
+        self.assertEqual(ref['service_id'], endpoint_v3['service_id'])
+        self.assertEqual(ref['description'], endpoint_v3['description'])
 
         # a v2 endpoint is not quite the same concept as a v3 endpoint, so they
         # receive different identifiers
         self.assertNotEqual(endpoint_v2['id'], endpoint_v3['id'])
 
         # v2 has a publicurl; v3 has a url + interface type
-        self.assertEqual(endpoint_v3['url'], ref['publicurl'])
-        self.assertEqual(endpoint_v3['interface'], 'public')
+        self.assertEqual(ref['publicurl'], endpoint_v3['url'])
+        self.assertEqual('public', endpoint_v3['interface'])
 
         # tests for bug 1152632 -- these attributes were being returned by v3
         self.assertNotIn('publicurl', endpoint_v3)
@@ -418,6 +627,8 @@ class CatalogTestCase(test_v3.RestfulTestCase):
         # test for bug 1152635 -- this attribute was being returned by v3
         self.assertNotIn('legacy_endpoint_id', endpoint_v3)
 
+        self.assertEqual(endpoint_v2['region'], endpoint_v3['region_id'])
+
 
 class TestCatalogAPISQL(tests.TestCase):
     """Tests for the catalog Manager against the SQL backend.
@@ -426,9 +637,7 @@ class TestCatalogAPISQL(tests.TestCase):
 
     def setUp(self):
         super(TestCatalogAPISQL, self).setUp()
-        self.load_backends()
-        self.load_fixtures(default_fixtures)
-
+        self.useFixture(database.Database())
         self.catalog_api = catalog.Manager()
 
         self.service_id = uuid.uuid4().hex
@@ -480,3 +689,72 @@ class TestCatalogAPISQL(tests.TestCase):
         self.assertEqual(1, len(catalog[0]['endpoints']))
         # all three appear in the backend
         self.assertEqual(3, len(self.catalog_api.list_endpoints()))
+
+
+# TODO(dstanek): this needs refactoring with the test above, but we are in a
+# crunch so that will happen in a future patch.
+class TestCatalogAPISQLRegions(tests.TestCase):
+    """Tests for the catalog Manager against the SQL backend.
+
+    """
+
+    def setUp(self):
+        super(TestCatalogAPISQLRegions, self).setUp()
+        self.useFixture(database.Database())
+        self.catalog_api = catalog.Manager()
+
+    def config_overrides(self):
+        super(TestCatalogAPISQLRegions, self).config_overrides()
+        self.config_fixture.config(
+            group='catalog',
+            driver='keystone.catalog.backends.sql.Catalog')
+
+    def new_endpoint_ref(self, service_id):
+        return {
+            'id': uuid.uuid4().hex,
+            'name': uuid.uuid4().hex,
+            'description': uuid.uuid4().hex,
+            'interface': uuid.uuid4().hex[:8],
+            'service_id': service_id,
+            'url': uuid.uuid4().hex,
+            'region_id': uuid.uuid4().hex,
+        }
+
+    def test_get_catalog_returns_proper_endpoints_with_no_region(self):
+        service_id = uuid.uuid4().hex
+        service = {'id': service_id, 'name': uuid.uuid4().hex}
+        self.catalog_api.create_service(service_id, service)
+
+        endpoint = self.new_endpoint_ref(service_id=service_id)
+        del endpoint['region_id']
+        self.catalog_api.create_endpoint(endpoint['id'], endpoint)
+
+        user_id = uuid.uuid4().hex
+        tenant_id = uuid.uuid4().hex
+
+        catalog = self.catalog_api.get_v3_catalog(user_id, tenant_id)
+        self.assertValidCatalogEndpoint(
+            catalog[0]['endpoints'][0], ref=endpoint)
+
+    def test_get_catalog_returns_proper_endpoints_with_region(self):
+        service_id = uuid.uuid4().hex
+        service = {'id': service_id, 'name': uuid.uuid4().hex}
+        self.catalog_api.create_service(service_id, service)
+
+        endpoint = self.new_endpoint_ref(service_id=service_id)
+        self.catalog_api.create_region({'id': endpoint['region_id']})
+        self.catalog_api.create_endpoint(endpoint['id'], endpoint)
+
+        endpoint = self.catalog_api.get_endpoint(endpoint['id'])
+        user_id = uuid.uuid4().hex
+        tenant_id = uuid.uuid4().hex
+
+        catalog = self.catalog_api.get_v3_catalog(user_id, tenant_id)
+        self.assertValidCatalogEndpoint(
+            catalog[0]['endpoints'][0], ref=endpoint)
+
+    def assertValidCatalogEndpoint(self, entity, ref=None):
+        keys = ['description', 'id', 'interface', 'name', 'region_id', 'url']
+        for k in keys:
+            self.assertEqual(ref.get(k), entity[k], k)
+        self.assertEqual(entity['region_id'], entity['region'])
