@@ -15,12 +15,13 @@
 import copy
 import functools
 
+from oslo.utils import timeutils
+
 from keystone.common import sql
 from keystone import config
 from keystone import exception
 from keystone.i18n import _LI
 from keystone.openstack.common import log
-from keystone.openstack.common import timeutils
 from keystone import token
 from keystone.token import provider
 
@@ -40,7 +41,9 @@ class TokenModel(sql.ModelBase, sql.DictBase):
     trust_id = sql.Column(sql.String(64))
     __table_args__ = (
         sql.Index('ix_token_expires', 'expires'),
-        sql.Index('ix_token_expires_valid', 'expires', 'valid')
+        sql.Index('ix_token_expires_valid', 'expires', 'valid'),
+        sql.Index('ix_token_user_id', 'user_id'),
+        sql.Index('ix_token_trust_id', 'trust_id')
     )
 
 
@@ -235,7 +238,7 @@ class Token(token.persistence.Driver):
     def _expiry_range_strategy(self, dialect):
         """Choose a token range expiration strategy
 
-        Based on the DB dialect, select a expiry range callable that is
+        Based on the DB dialect, select an expiry range callable that is
         appropriate.
         """
 
@@ -266,7 +269,7 @@ class Token(token.persistence.Driver):
         total_removed = 0
         upper_bound_func = timeutils.utcnow
         for expiry_time in expiry_range_func(session, upper_bound_func):
-            delete_query = query.filter(TokenModel.expires <
+            delete_query = query.filter(TokenModel.expires <=
                                         expiry_time)
             row_count = delete_query.delete(synchronize_session=False)
             total_removed += row_count
