@@ -12,9 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import json
-import sys
-
+from oslo.utils import timeutils
 import six
 from six.moves.urllib import parse
 
@@ -22,15 +20,13 @@ from keystone.common import dependency
 from keystone import config
 from keystone.contrib import federation
 from keystone import exception
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
+from keystone.openstack.common import jsonutils
+from keystone.openstack.common import log
 from keystone import token
 from keystone.token import provider
 
 
-from keystone.openstack.common import log
-from keystone.openstack.common import timeutils
-
-
 LOG = log.getLogger(__name__)
 CONF = config.CONF
 
@@ -38,18 +34,31 @@ CONF = config.CONF
 class V2TokenDataHelper(object):
     """Creates V2 token data."""
     @classmethod
-    def format_token(cls, token_ref, roles_ref=None, catalog_ref=None):
+    def format_token(cls, token_ref, roles_ref=None, catalog_ref=None,
+                     trust_ref=None):
+        audit_info = None
         user_ref = token_ref['user']
         metadata_ref = token_ref['metadata']
         if roles_ref is None:
             roles_ref = []
-        expires = token_ref.get('expires', token.default_expire_time())
+        expires = token_ref.get('expires', provider.default_expire_time())
         if expires is not None:
             if not isinstance(expires, six.text_type):
                 expires = timeutils.isotime(expires)
+
+        token_data = token_ref.get('token_data')
+        if token_data:
+            token_audit = token_data.get(
+                'access', token_data).get('token', {}).get('audit_ids')
+            audit_info = token_audit
+
+        if audit_info is None:
+            audit_info = provider.audit_info(token_ref.get('parent_audit_id'))
+
         o = {'access': {'token': {'id': token_ref['id'],
                                   'expires': expires,
-                                  'issued_at': timeutils.strtime()
+                                  'issued_at': timeutils.strtime(),
+                                  'audit_ids': audit_info
                                   },
                         'user': {'id': user_ref['id'],
                                  'name': user_ref['name'],
@@ -76,10 +85,14 @@ class V2TokenDataHelper(object):
                 o['access']['metadata'] = {'is_admin': 0}
         if 'roles' in metadata_ref:
             o['access']['metadata']['roles'] = metadata_ref['roles']
-        if CONF.trust.enabled and 'trust_id' in metadata_ref:
+        if CONF.trust.enabled and trust_ref:
             o['access']['trust'] = {'trustee_user_id':
-                                    metadata_ref['trustee_user_id'],
-                                    'id': metadata_ref['trust_id']
+                                    trust_ref['trustee_user_id'],
+                                    'id': trust_ref['id'],
+                                    'trustor_user_id':
+                                    trust_ref['trustor_user_id'],
+                                    'impersonation':
+                                    trust_ref['impersonation']
                                     }
         return o
 
@@ -208,7 +221,9 @@ class V3TokenDataHelper(object):
         if CONF.trust.enabled and trust and 'OS-TRUST:trust' not in token_data:
             trustor_user_ref = (self.identity_api.get_user(
                                 trust['trustor_user_id']))
-            if not trustor_user_ref['enabled']:
+            try:
+                self.identity_api.assert_user_enabled(trust['trustor_user_id'])
+            except AssertionError:
                 raise exception.Forbidden(_('Trustor is disabled.'))
             if trust['impersonation']:
                 user_ref = trustor_user_ref
@@ -240,7 +255,7 @@ class V3TokenDataHelper(object):
 
         if access_token:
             filtered_roles = []
-            authed_role_ids = json.loads(access_token['role_ids'])
+            authed_role_ids = jsonutils.loads(access_token['role_ids'])
             all_roles = self.assignment_api.list_roles()
             for role in all_roles:
                 for authed_role in authed_role_ids:
@@ -253,7 +268,7 @@ class V3TokenDataHelper(object):
         if CONF.trust.enabled and trust:
             token_user_id = trust['trustor_user_id']
             token_project_id = trust['project_id']
-            #trusts do not support domains yet
+            # trusts do not support domains yet
             token_domain_id = None
         else:
             token_user_id = user_id
@@ -313,17 +328,29 @@ class V3TokenDataHelper(object):
     def _populate_token_dates(self, token_data, expires=None, trust=None,
                               issued_at=None):
         if not expires:
-            expires = token.default_expire_time()
+            expires = provider.default_expire_time()
         if not isinstance(expires, six.string_types):
             expires = timeutils.isotime(expires, subsecond=True)
         token_data['expires_at'] = expires
         token_data['issued_at'] = (issued_at or
                                    timeutils.isotime(subsecond=True))
 
+    def _populate_audit_info(self, token_data, audit_info=None):
+        if audit_info is None or isinstance(audit_info, six.string_types):
+            token_data['audit_ids'] = provider.audit_info(audit_info)
+        elif isinstance(audit_info, list):
+            token_data['audit_ids'] = audit_info
+        else:
+            msg = _('Invalid audit info data type: %(data)s (%(type)s)')
+            msg_subst = {'data': audit_info, 'type': type(audit_info)}
+            LOG.error(msg, msg_subst)
+            raise exception.UnexpectedError(msg % msg_subst)
+
     def get_token_data(self, user_id, method_names, extras,
                        domain_id=None, project_id=None, expires=None,
                        trust=None, token=None, include_catalog=True,
-                       bind=None, access_token=None, issued_at=None):
+                       bind=None, access_token=None, issued_at=None,
+                       audit_info=None):
         token_data = {'methods': method_names,
                       'extras': extras}
 
@@ -344,6 +371,8 @@ class V3TokenDataHelper(object):
         self._populate_user(token_data, user_id, trust)
         self._populate_roles(token_data, user_id, domain_id, project_id, trust,
                              access_token)
+        self._populate_audit_info(token_data, audit_info)
+
         if include_catalog:
             self._populate_service_catalog(token_data, user_id, domain_id,
                                            project_id, trust)
@@ -353,9 +382,9 @@ class V3TokenDataHelper(object):
         return {'token': token_data}
 
 
-@dependency.optional('oauth_api', 'revoke_api')
+@dependency.optional('oauth_api')
 @dependency.requires('assignment_api', 'catalog_api', 'identity_api',
-                     'token_api', 'trust_api')
+                     'trust_api')
 class BaseProvider(provider.Provider):
     def __init__(self, *args, **kwargs):
         super(BaseProvider, self).__init__(*args, **kwargs)
@@ -375,44 +404,25 @@ class BaseProvider(provider.Provider):
                 return token.provider.V2
             if 'token' in token_data and 'methods' in token_data['token']:
                 return token.provider.V3
-        raise token.provider.UnsupportedTokenVersionException()
+        raise exception.UnsupportedTokenVersionException()
 
     def issue_v2_token(self, token_ref, roles_ref=None,
                        catalog_ref=None):
+        metadata_ref = token_ref['metadata']
+        trust_ref = None
+        if CONF.trust.enabled and metadata_ref and 'trust_id' in metadata_ref:
+            trust_ref = self.trust_api.get_trust(metadata_ref['trust_id'])
+
         token_data = self.v2_token_data_helper.format_token(
-            token_ref, roles_ref, catalog_ref)
+            token_ref, roles_ref, catalog_ref, trust_ref)
         token_id = self._get_token_id(token_data)
         token_data['access']['token']['id'] = token_id
-        try:
-            expiry = token_data['access']['token']['expires']
-            if isinstance(expiry, six.string_types):
-                expiry = timeutils.normalize_time(
-                    timeutils.parse_isotime(expiry))
-            data = dict(key=token_id,
-                        id=token_id,
-                        expires=expiry,
-                        user=token_ref['user'],
-                        tenant=token_ref['tenant'],
-                        metadata=token_ref['metadata'],
-                        token_data=token_data,
-                        bind=token_ref.get('bind'),
-                        trust_id=token_ref['metadata'].get('trust_id'),
-                        token_version=token.provider.V2)
-            self.token_api.create_token(token_id, data)
-        except Exception:
-            exc_info = sys.exc_info()
-            # an identical token may have been created already.
-            # if so, return the token_data as it is also identical
-            try:
-                self.token_api.get_token(token_id)
-            except exception.TokenNotFound:
-                raise exc_info[0], exc_info[1], exc_info[2]
-
-        return (token_id, token_data)
+        return token_id, token_data
 
     def issue_v3_token(self, user_id, method_names, expires_at=None,
                        project_id=None, domain_id=None, auth_context=None,
-                       trust=None, metadata_ref=None, include_catalog=True):
+                       trust=None, metadata_ref=None, include_catalog=True,
+                       parent_audit_id=None):
         # for V2, trust is stashed in metadata_ref
         if (CONF.trust.enabled and not trust and metadata_ref and
                 'trust_id' in metadata_ref):
@@ -442,55 +452,25 @@ class BaseProvider(provider.Provider):
             bind=auth_context.get('bind') if auth_context else None,
             token=token_ref,
             include_catalog=include_catalog,
-            access_token=access_token)
+            access_token=access_token,
+            audit_info=parent_audit_id)
 
         token_id = self._get_token_id(token_data)
-        try:
-            expiry = token_data['token']['expires_at']
-            if isinstance(expiry, six.string_types):
-                expiry = timeutils.normalize_time(
-                    timeutils.parse_isotime(expiry))
-            # FIXME(gyee): is there really a need to store roles in metadata?
-            role_ids = []
-            if metadata_ref is None:
-                metadata_ref = {}
-            if 'project' in token_data['token']:
-                # project-scoped token, fill in the v2 token data
-                # all we care are the role IDs
-                role_ids = [r['id'] for r in token_data['token']['roles']]
-                metadata_ref = {'roles': role_ids}
-            if trust:
-                metadata_ref.setdefault('trust_id', trust['id'])
-                metadata_ref.setdefault('trustee_user_id',
-                                        trust['trustee_user_id'])
-            data = dict(key=token_id,
-                        id=token_id,
-                        expires=expiry,
-                        user=token_data['token']['user'],
-                        tenant=token_data['token'].get('project'),
-                        metadata=metadata_ref,
-                        token_data=token_data,
-                        trust_id=trust['id'] if trust else None,
-                        token_version=token.provider.V3)
-            self.token_api.create_token(token_id, data)
-        except Exception:
-            exc_info = sys.exc_info()
-            # an identical token may have been created already.
-            # if so, return the token_data as it is also identical
-            try:
-                self.token_api.get_token(token_id)
-            except exception.TokenNotFound:
-                raise exc_info[0], exc_info[1], exc_info[2]
-
-        return (token_id, token_data)
+        return token_id, token_data
 
     def _handle_saml2_tokens(self, auth_context, project_id, domain_id):
         user_id = auth_context['user_id']
         group_ids = auth_context['group_ids']
+        idp = auth_context[federation.IDENTITY_PROVIDER]
+        protocol = auth_context[federation.PROTOCOL]
         token_data = {
             'user': {
                 'id': user_id,
-                'name': parse.unquote(user_id)
+                'name': parse.unquote(user_id),
+                federation.FEDERATION: {
+                    'identity_provider': {'id': idp},
+                    'protocol': {'id': protocol}
+                }
             }
         }
 
@@ -499,51 +479,17 @@ class BaseProvider(provider.Provider):
                 group_ids, project_id, domain_id, user_id)
             token_data.update({'roles': roles})
         else:
-            idp = auth_context[federation.IDENTITY_PROVIDER]
-            protocol = auth_context[federation.PROTOCOL]
-            token_data['user'].update({
-                federation.FEDERATION: {
-                    'identity_provider': {'id': idp},
-                    'protocol': {'id': protocol},
-                    'groups': [{'id': x} for x in group_ids]
-                },
+            token_data['user'][federation.FEDERATION].update({
+                'groups': [{'id': x} for x in group_ids]
             })
         return token_data
 
-    def _verify_token(self, token_id):
-        """Verify the given token and return the token_ref."""
-        token_ref = self.token_api.get_token(token_id)
-        return self._verify_token_ref(token_ref)
-
     def _verify_token_ref(self, token_ref):
         """Verify and return the given token_ref."""
         if not token_ref:
             raise exception.Unauthorized()
         return token_ref
 
-    def revoke_token(self, token_id):
-        token = self.token_api.get_token(token_id)
-        if self.revoke_api:
-            version = self.get_token_version(token)
-            if version == provider.V3:
-                user_id = token['user']['id']
-                expires_at = token['expires']
-
-                token_data = token['token_data']['token']
-                project_id = token_data.get('project', {}).get('id')
-                domain_id = token_data.get('domain', {}).get('id')
-            elif version == provider.V2:
-                user_id = token['user_id']
-                expires_at = token['expires']
-                project_id = (token.get('tenant') or {}).get('id')
-                domain_id = None  # A V2 token can't be scoped to a domain.
-            self.revoke_api.revoke_by_expiration(user_id, expires_at,
-                                                 project_id=project_id,
-                                                 domain_id=domain_id)
-
-        if CONF.token.revoke_by_id:
-            self.token_api.delete_token(token_id=token_id)
-
     def _assert_default_domain(self, token_ref):
         """Make sure we are operating on default domain only."""
         if (token_ref.get('token_data') and
@@ -588,11 +534,7 @@ class BaseProvider(provider.Provider):
                         CONF.identity.default_domain_id):
                     raise exception.Unauthorized(msg)
 
-    def validate_v2_token(self, token_id):
-        token_ref = self._verify_token(token_id)
-        return self._validate_v2_token_ref(token_ref)
-
-    def _validate_v2_token_ref(self, token_ref):
+    def validate_v2_token(self, token_ref):
         try:
             self._assert_default_domain(token_ref)
             # FIXME(gyee): performance or correctness? Should we return the
@@ -621,22 +563,20 @@ class BaseProvider(provider.Provider):
                         token_ref['user']['id'],
                         token_ref['tenant']['id'],
                         metadata_ref)
+
+                trust_ref = None
+                if CONF.trust.enabled and 'trust_id' in metadata_ref:
+                    trust_ref = self.trust_api.get_trust(
+                        metadata_ref['trust_id'])
+
                 token_data = self.v2_token_data_helper.format_token(
-                    token_ref, roles_ref, catalog_ref)
+                    token_ref, roles_ref, catalog_ref, trust_ref)
             return token_data
         except exception.ValidationError as e:
             LOG.exception(_('Failed to validate token'))
             raise exception.TokenNotFound(e)
 
-    def validate_v3_token(self, token_id):
-        try:
-            token_ref = self._verify_token(token_id)
-            token_data = self._validate_v3_token_ref(token_ref)
-            return token_data
-        except (exception.ValidationError, exception.UserNotFound):
-            raise exception.TokenNotFound(token_id)
-
-    def _validate_v3_token_ref(self, token_ref):
+    def validate_v3_token(self, token_ref):
         # FIXME(gyee): performance or correctness? Should we return the
         # cached token or reconstruct it? Obviously if we are going with
         # the cached token, any role, project, or domain name changes
@@ -654,6 +594,7 @@ class BaseProvider(provider.Provider):
                 project_id = project_ref['id']
 
             issued_at = token_ref['token_data']['access']['token']['issued_at']
+            audit = token_ref['token_data']['access']['token'].get('audit_ids')
 
             token_data = self.v3_token_data_helper.get_token_data(
                 token_ref['user']['id'],
@@ -662,14 +603,6 @@ class BaseProvider(provider.Provider):
                 project_id=project_id,
                 bind=token_ref.get('bind'),
                 expires=token_ref['expires'],
-                issued_at=issued_at)
+                issued_at=issued_at,
+                audit_info=audit)
         return token_data
-
-    def validate_token(self, token_id):
-        token_ref = self._verify_token(token_id)
-        version = self.get_token_version(token_ref)
-        if version == token.provider.V3:
-            return self._validate_v3_token_ref(token_ref)
-        elif version == token.provider.V2:
-            return self._validate_v2_token_ref(token_ref)
-        raise token.provider.UnsupportedTokenVersionException()
