@@ -10,32 +10,25 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""Keystone Compressed PKI Token Provider."""
-
-import subprocess  # nosec : used to catch subprocess exceptions
+"""Keystone Compressed PKI Token Provider"""
 
 from keystoneclient.common import cms
-from oslo_log import log
-from oslo_log import versionutils
-from oslo_serialization import jsonutils
 
-from keystone.common import utils
-import keystone.conf
+from keystone.common import environment
+from keystone import config
 from keystone import exception
 from keystone.i18n import _
+from keystone.openstack.common import jsonutils
+from keystone.openstack.common import log
 from keystone.token.providers import common
 
 
-CONF = keystone.conf.CONF
+CONF = config.CONF
 
 LOG = log.getLogger(__name__)
 ERROR_MESSAGE = _('Unable to sign token.')
 
 
-@versionutils.deprecated(
-    as_of=versionutils.deprecated.MITAKA,
-    what='the PKIZ token provider',
-    in_favor_of='the Fernet or UUID token providers')
 class Provider(common.BaseProvider):
     def _get_token_id(self, token_data):
         try:
@@ -43,23 +36,10 @@ class Provider(common.BaseProvider):
             # produces unicode. This can be removed if the client returns
             # str()
             # TODO(ayoung): Make to a byte_str for Python3
-            token_json = jsonutils.dumps(token_data, cls=utils.PKIEncoder)
-            token_id = str(cms.pkiz_sign(token_json,
+            token_id = str(cms.pkiz_sign(jsonutils.dumps(token_data),
                                          CONF.signing.certfile,
                                          CONF.signing.keyfile))
             return token_id
-        except subprocess.CalledProcessError:
+        except environment.subprocess.CalledProcessError:
             LOG.exception(ERROR_MESSAGE)
             raise exception.UnexpectedError(ERROR_MESSAGE)
-
-    @property
-    def _supports_bind_authentication(self):
-        """Return if the token provider supports bind authentication methods.
-
-        :returns: True
-        """
-        return True
-
-    def needs_persistence(self):
-        """Should the token be written to a backend."""
-        return True
